
/*****************************************************************************
 *
 *  XviD Standard Plugins
 *  - external bitrate controller implementation -
 *
 *  Copyright(C) 2007 Kopernikus <kopernikus@gleitz.info>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: plugin_single.c,v 1.3 2004/08/10 22:34:32 edgomez Exp $
 *
 ****************************************************************************/

#include <limits.h>
#include <stdio.h>

#include "../xvid.h"
#include "../image/image.h"

typedef struct
{
	int num_frames;
	int* type;
	int* quant;
	int* vol_flags;
	int* vop_flags;
	int* motion_flags;	
}
rc_external_t;

static int
rc_external_create(xvid_plg_create_t * create,
			  rc_external_t ** handle)
{
	xvid_plugin_external_t *param = (xvid_plugin_external_t *) create->param;
	rc_external_t *rc;
	int i=0,num_frames=0, quant=0,type=0,vol=0,vop=0,motion=0;

	FILE* infile;

	if((infile = fopen(param->filename,"r")) == NULL){
		printf("Error in plugin_external.c: Cannot open file\n");
		return XVID_ERR_FAIL; 
	}

	/*scan the file to find number of specified frames*/
	while(fscanf(infile,"%d %d %d %d %d\n",&type,&quant,&vol,&vop,&motion) != EOF) num_frames++;

	/*rewind*/
	if(fseek(infile,0,SEEK_SET) != 0){
		printf("Error in plugin_external.c: Cannot seek in file\n");
		return XVID_ERR_FAIL;
	}

	fprintf(stderr,"Read Information for %d frames\n",num_frames);

	/*allocate space in rc*/
	rc = (rc_external_t*) malloc(sizeof(rc_external_t)); 

	if(rc == NULL) return XVID_ERR_MEMORY;

	rc->type = (int*) malloc(sizeof(int)*num_frames);
	if(rc->type == NULL) goto err_type;
		
	rc->quant = (int*) malloc(sizeof(int)*num_frames);
	if(rc->type == NULL) goto err_quant;

	rc->vol_flags = (int*) malloc(sizeof(int)*num_frames);
	if(rc->type == NULL) goto err_vol;

	rc->vop_flags = (int*) malloc(sizeof(int)*num_frames);
	if(rc->type == NULL) goto err_vop;

	rc->motion_flags = (int*) malloc(sizeof(int)*num_frames);
	if(rc->type == NULL) goto err_motion;

	/*fill rc*/

	rc->num_frames = num_frames;

	for(i=0;i<num_frames;i++){
		fscanf(infile,"%d %d %d %d %d\n",&type,&quant,&vol,&vop,&motion);

		rc->type[i] = type;
		rc->quant[i] = quant;
		rc->vol_flags[i] = vol;
		rc->vop_flags[i] = vop;
		rc->motion_flags[i] = motion;
	}

	/* Bind the RC */
	*handle = rc;



	/*success*/
	fclose(infile);
	return (0);

	/*Error handling*/
	err_motion:
		free(rc->vop_flags);
	err_vop:
		free(rc->vol_flags);
	err_vol:
		free(rc->quant);
	err_quant:
		free(rc->type);
	err_type:
		free(rc);

	printf("Error in plugin_external.c: Error allocating memory\n");

	fclose(infile);
	return XVID_ERR_MEMORY;

}


static int
rc_external_destroy(rc_external_t * rc,
			   xvid_plg_destroy_t * destroy)
{
	free(rc->motion_flags);
	free(rc->vop_flags);
	free(rc->vol_flags);
	free(rc->quant);
	free(rc->type);
	free(rc);
	return 0;
}


static int
rc_external_before(rc_external_t * rc,
			  xvid_plg_data_t * data)
{
	int fn = data->frame_num;

	if(fn > rc->num_frames){
		printf("Warning in plugin_external.c: Inputfile ends too early");
		return 0;
	}

	data->type = rc->type[fn];

	if(rc->quant[fn] > 31 || rc->quant[fn]< 1){
		printf("Error in plugin_external.c: Invalid Quantizer\n");
		return XVID_ERR_FAIL;
	}

	/*TODO: More checks*/
	data->quant = rc->type[fn];
	data->vop_flags = rc->vop_flags[fn];
	data->vol_flags = rc->vol_flags[fn];
	data->motion_flags = rc->motion_flags[fn];
	
	return 0;
}

int
xvid_plugin_external(void *handle,
				int opt,
				void *param1,
				void *param2)
{
	switch (opt) {
	case XVID_PLG_INFO:
	case XVID_PLG_FRAME :
	case XVID_PLG_AFTER:
		return 0;

	case XVID_PLG_CREATE:
		return rc_external_create((xvid_plg_create_t *) param1, param2);

	case XVID_PLG_DESTROY:
		return rc_external_destroy((rc_external_t *) handle,(xvid_plg_destroy_t *) param1);

	case XVID_PLG_BEFORE:
		return rc_external_before((rc_external_t *) handle, (xvid_plg_data_t *) param1);

	}

	return XVID_ERR_FAIL;
}
